package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.Collection;

/**
 * Protocol according to which the packages are created. Can contain different
 * Roles.
 * 
 * @author Andreas T. Meyer-Berg
 */
public interface Protocol {

	/**
	 * Generates the next packet
	 * 
	 * @param timestep
	 *            Time the package should be sent, in System.currentTimeMillis
	 * @return next Packet, which was sent
	 */
	public Packet generateNextPaket(long timestep);

	/**
	 * Returns the number of different roles the participating SmartDevices
	 * could have
	 * 
	 * @return number of different roles
	 */
	public int getNumberOfRoles();

	/**
	 * Returns the textual representation of the different Roles. The Array
	 * should contain {@code NumberOfRoles} Strings. {@code Array[role]} should be
	 * a human-readable String representation of the Role at its position
	 * {@code role}.
	 * 
	 * @return String representations of the roles
	 */
	public String[] getRoles();

	/**
	 * Returns all SmartDevices of the given Role. Returns {@code null}, if the role
	 * number was invalid.
	 * 
	 * @param role
	 *            Position of the role in {@code getNumberOfRoles}
	 * @return SmartDevices of Role with index {@code role}
	 */
	public Collection<SmartDevice> getDevicesWithRole(int role);

	/**
	 * Adds a new SmartDevice to the role, returns {@code true} if it was
	 * assigned successfully, {@code false} if it wasn't. (Either invalid role
	 * number or maximum number of devices for the role reached)
	 * 
	 * @param device
	 *            SmartDevice that should be assigned to the given role
	 * @param role
	 *            Position of the role in {@code getNumberOfRoles}
	 * @return true, if the SmartDevice was added
	 */
	public boolean addDeviceOfRole(SmartDevice device, int role);

	/**
	 * Remove a SmartDevice from this protocol
	 * 
	 * @param device
	 *            device that should be removed
	 */
	public void removeDevice(SmartDevice device);
}
